Ext.define('DSS.component.DeviceSelector', {
    extend : 'DSS.component.Base',
    mixins : {
        observable : 'Ext.util.Observable'
    },
    height : '100%',
    grids : {},
    macPanel : null,
    iDefaultWidth : 150,
    iDefaultHeight : 293,
    filterFunc:null,
    specialDeviceRenderer:null,
    selectedDSUID:null,
    myID:0,
    bMultiselect:false,
    setHeight : function(oHeight) {
        this.macPanel.setHeight(oHeight);
        this.iDefaultHeight = oHeight - 2;
        for ( var oKey in this.grids)
            this.grids[oKey].setHeight(this.iDefaultHeight);
        this.callParent(arguments);
    },
    initStores : function() {
        if (Ext.data.StoreManager.lookup('deviceSelector_zoneStore'+this.myID) == null) {
            
            Ext.create('Ext.data.Store', {
                storeId : 'deviceSelector_zoneStore' + this.myID,
                fields : [ 'ZoneID', 'name' , 'passive'],
                proxy : {
                    type : 'memory',
                    reader : {
                        type : 'json',
                        root : 'items'
                    }
                }
            });
            var oArray=dss.staticDataModel.getZoneArray();
            
            for (var iIndex=0;iIndex<oArray.length;iIndex++) {
                Ext.create('Ext.data.Store', {
                    storeId : 'deviceSelector_deviceStore_' + oArray[iIndex].ZoneID + this.myID,
                    fields : [ 'dSID', 'dSUID', 'name', 'icon', 'present' ],
                    proxy : {
                        type : 'memory',
                        reader : {
                            type : 'json',
                            root : 'items'
                        }
                    }
                });
                var oObject={
                        ZoneID:oArray[iIndex].ZoneID,
                        name:oArray[iIndex].name,
                        passive:true
                        };
                
                var oDeviceArray=dss.staticDataModel.getDevicesForZone(oArray[iIndex].ZoneID,0);
                for (var jIndex=0;jIndex<oDeviceArray.length;jIndex++) {
                    var oDeviceObject=oDeviceArray[jIndex];
                    if (oDeviceObject.present) {
                        if (this.filterFunc==null) {
                            oObject.passive = false;                            
                            Ext.data.StoreManager.lookup('deviceSelector_deviceStore_' + oArray[iIndex].ZoneID + this.myID).add(oDeviceObject);
                        } else
                            if (this.filterFunc(oDeviceArray[jIndex])) {
                                oObject.passive = false;
                                Ext.data.StoreManager.lookup('deviceSelector_deviceStore_' + oArray[iIndex].ZoneID + this.myID).add(oDeviceObject);
                            }
                    }
                }
                Ext.data.StoreManager.lookup('deviceSelector_zoneStore' + this.myID).add(oObject);
            }
        }
    },   
    defMacGridRenderer : function(val, metadata, record, colIdx, store, view) {
        var fHasIcon=false;
        if (record.get('icon') != null)
        {
            fHasIcon=true;
        }
        var fHasNext=true;
        if ((record.get('dSID') != null) || (record.get('dSUID') != null))   // ist ein device -> kein weiterbutton
        {
            fHasNext=false;
        } 

        var fIsPassive=false;        
        if (record.get('passive')!=null)
            if (record.get('passive'))
                fIsPassive=true;
        
        if (fIsPassive) {
            fHasNext=false;               // spalte disabled -> niemals weiterbutton
        }
        
        var sReturn='<table style="height:16px;width:100%"><tr>';
        var iColspaceCounter=3;
        if (fHasIcon)
        {
            iColspaceCounter--;  
            sReturn+='<td><img src="' + record.get('icon') + '" style="margin-right: 5px;"></td>';
        }

        if (fHasNext)
            iColspaceCounter--;
        if (fIsPassive) {
            sReturn+='<td style="color:#888888;" colspan="'+iColspaceCounter+'" width="100%">' + record.get('name') + '</td>';
        } else {
            sReturn+='<td colspan="'+iColspaceCounter+'" width="100%">' + record.get('name') + '</td>';
        }
        if (fHasNext) {
            sReturn+='<td>&gt;</td>';
        }
        return sReturn +'</tr></table>';      
    },
    getDeviceStore : function (zoneID) {
        return Ext.data.StoreManager.lookup('deviceSelector_deviceStore_' + zoneID + this.myID);
    },
    constructGridLvlDevice : function (zoneID) {
        this.selectedDSUID=null;
        // construct device grids
        var oDeviceControlStore = Ext.create('Ext.data.Store', {
            fields : [ 'dSID', 'dSUID', 'name', 'icon', 'present' ],
            proxy : {
                type : 'memory',
                reader : {
                    type : 'json',
                    root : 'items'
                }
            }
        });

        var oDeviceStore = this.getDeviceStore(zoneID);
        oDeviceStore.each(function(record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('present'))
                oDeviceControlStore.add(oNewRecord);
        });
        oDeviceControlStore.sort('name');
        this.grids.lvlDevice = Ext.create('Ext.grid.Panel', {
            hideHeaders:true,
            height : this.iDefaultHeight,
            width : 624,
            autoScroll : true,
            viewConfig : {
                stripeRows : true
            },
            store : oDeviceControlStore,
            columns : [ {
                sortable : false,
                hideable : false,
                text : _('device'),
                dataIndex : 'name',
                flex : 1,
                renderer : (this.specialDeviceRenderer!=null)?this.specialDeviceRenderer:this.defMacGridRenderer
            } ]
        });
        if (this.bMultiselect) {
            this.grids.lvlDevice.getSelectionModel().setSelectionMode('MULTI');
        }
        this.grids.lvlDevice.parent = this;
        this.grids.lvlDevice.on('itemclick', function(grid, record) {
            this.parent.selectedDSUID=record.get('dSUID');
        });
        this.macPanel.add(this.grids.lvlDevice);

    
    },
    constructMacGrids : function() {
        var oZoneStore;
        // construct room grid
        if (this.grids.lvlRoom==null) {
	        var oZoneControlStore = Ext.create('Ext.data.Store', {
	            fields : [ 'ZoneID', 'name' , 'passive'],
	            proxy : {
	                type : 'memory',
	                reader : {
	                    type : 'json',
	                    root : 'items'
	                }
	            }
	        });
	
	        oZoneStore = Ext.data.StoreManager.lookup('deviceSelector_zoneStore'+this.myID);
	        oZoneStore.each(function(record) {
	            var oNewRecord = record.copy();
	            if (oNewRecord.get('ZoneID') != 0)
	                oZoneControlStore.add(oNewRecord);
	        });
	        oZoneControlStore.sort('name');
	
	        this.grids.lvlRoom = Ext.create('Ext.grid.Panel', {
	            hideHeaders:true,
	            height : this.iDefaultHeight,
	            width : this.iDefaultWidth,
	            autoScroll : true,
	            viewConfig : {
	                stripeRows : true
	            },
	            store : oZoneControlStore,
	            columns : [ {
	                sortable : false,
	                hideable : false,
	                text : _('room'),
	                dataIndex : 'name',
	                flex : 1,
	                renderer :function(val, metadata, record, colIdx, store, view) {
	                    var fIsPassive=false;        
	                    if (record.get('passive')!=null)
	                        if (record.get('passive'))
	                            fIsPassive=true;
	                    
	                    var sReturn='<table style="height:16px;width:100%"><tr>';
	                    if (fIsPassive) {
	                        sReturn+='<td style="color:#888888;" colspan="2" width="100%">' + record.get('name') + '</td>';
	                    } else {
	                        sReturn+='<td width="100%">' + record.get('name') + '</td><td>&gt;</td>';
	                    }
	                    return sReturn +'</tr></table>';      
	                }
	            } ]
	        });
	
	        this.grids.lvlRoom.parent = this;
	        this.grids.lvlRoom.on('itemclick', function(grid, record) {
	            this.up().remove(this.up().up().grids.lvlDevice);
	            var zoneID = record.get('ZoneID');
	            if (zoneID == -1)
	                zoneID = 0;
	            this.up().up().constructGridLvlDevice(zoneID);
	        });
	        this.macPanel.add(this.grids.lvlRoom);
        } else {
        	this.grids.lvlRoom.getStore().removeAll();
        	var mThis=this;        	
	        oZoneStore = Ext.data.StoreManager.lookup('deviceSelector_zoneStore'+this.myID);
	        oZoneStore.each(function(record) {
	            var oNewRecord = record.copy();
	            if (oNewRecord.get('ZoneID') != 0)
	            	mThis.grids.lvlRoom.getStore().add(oNewRecord);
	        });	              	
        }
    },
    constructor : function (config) {
        this.macPanel = null;
        this.selectedDSUID=null;
    	
    	this.grids={};
        if (config.filterFunc!=null)
            this.filterFunc=config.filterFunc;
        else
            this.filterFunc=null;
        if (config.specialDeviceRenderer!=null)
            this.specialDeviceRenderer=config.specialDeviceRenderer;        
        else
            this.specialDeviceRenderer=null;
        if (config.myID==null)
        	this.myID=""+Math.random();
        else
        	this.myID=config.myID;
        if (config.bMultiselect != null) {
            this.bMultiselect = config.bMultiselect;
        }
        this.initStores();
        var mThis=this;
        Ext.data.StoreManager.on('clear',function () {mThis.initStores();mThis.constructMacGrids();});
        this.macPanel = Ext.create('Ext.panel.Panel', {
            border : '0',
            autoScroll : true,
            height : 320,
            layout : {
                type : 'hbox'
            },
            scroll : 'horizontal'
        });
        this.constructMacGrids();
        Ext.apply(this, {
            items : [ this.macPanel ]
        });

        this.callParent(arguments);
  
    },
    reset : function() {
        this.setValue(null);
    },
    setValue : function(values) {
        this.macPanel.remove(this.grids.lvlDevice);
        this.selectedDSUID=null; //reset first
        if (values==null)
        	 this.grids.lvlRoom.getSelectionModel().deselectAll();
        var oDevice=dss.staticDataModel.getDevice(values);
        var oRecordZone = this.grids.lvlRoom.store.find('ZoneID', oDevice.ZoneID,0,false,false,true);
        if (oRecordZone != -1) {
            this.grids.lvlRoom.getSelectionModel().select(oRecordZone);
            this.constructGridLvlDevice(oDevice.ZoneID);
            
            var oRecordDevice = this.grids.lvlDevice.store.find('dSUID', values,0,false,false,true);
            if (oRecordDevice != -1) {
                this.grids.lvlDevice.getSelectionModel().select(oRecordDevice);
                this.selectedDSUID=values;
            }
        }
    },
    isValid : function() {
        return (this.selectedDSUID!=null);
    },
    getValue : function() {
        if (this.bMultiselect) {
            if ( this.grids.lvlDevice) {
                var recordList = this.grids.lvlDevice.getSelectionModel().getSelection();
                var dSUID = [];
                for (var jIndex = 0; jIndex < recordList.length; jIndex++) {
                    dSUID.push(recordList[jIndex].get('dSUID'));
                }
                return dSUID;
            } else {
                return null;
            }
        } else {
            return this.selectedDSUID;
        }
    }
});